﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Infragistics.Win.UltraWinGrid;
using System.Drawing;
using System.Windows.Forms;
using Infragistics.Win;

namespace WNG_DragAndDropRowIndicator
{
    public class WinGridDragDropRow : UltraGrid
    {
        #region Fields

        private DragDropRowDrawFilter dFilter = new DragDropRowDrawFilter();
        
        // stores a boolean that will determine when the mouse is moved if there is something to drag.
        bool allowDrag = false;

        // stores the item to drag
        UltraGridRow rowToDrag = null;

        // stores the point that the initial mouse down occurred
        Point startPoint;
        #endregion

        #region Events
        void dFilter_Invalidate(object sender, EventArgs e)
        {
            Invalidate();
        }

        void WinGridDragDropRow_MouseDown(object sender, MouseEventArgs e)
        {
            //get the rowElement from the mouse location
            Point pointInGrid = new Point(e.X, e.Y);
            UIElement element = this.DisplayLayout.UIElement.ElementFromPoint(pointInGrid);
            UltraGridRow row = null;

            //get row from element
            if (element != null)
            {
                RowUIElement rowElement = element.GetAncestor(typeof(RowUIElement)) as RowUIElement;
                if (rowElement != null)
                {
                    row = rowElement.Row;
                }
            }

            //if there is a row here, we have to store for possible drag and drop
            if (row != null)
            {
                //store the row
                rowToDrag = row;
                //set the flag so dragging can be done in mouse move
                allowDrag = true;
                //store the start point
                startPoint = new Point(e.X, e.Y);
            }
        }

        void WinGridDragDropRow_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e)
        {
            // Once the mouse comes up, we no longer want to start a drag operation.
            allowDrag = false;
        }

        void WinGridDragDropRow_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
        {
            // check if we have set the flag for allowing dragging (indicates that the mouse has gone 
            // down over an element and is still down and a drag drop hasn't yet started)
            if (allowDrag)
            {
                Point gridPoint = new Point(e.X, e.Y);

                // check if we have moved a few pixels
                if (DragDistanceReached(startPoint, gridPoint))
                {
                    // we have moved a vew pixels so we will begin the drag, AllowDrag flag is reset
                    allowDrag = false;
                    //Start drag and set indicator
                    dFilter.SetDropHighlight(rowToDrag, gridPoint);
                    DoDragDrop(rowToDrag, DragDropEffects.Move);
                }
            }
        }

        void WinGridDragDropRow_MouseLeave(object sender, EventArgs e)
        {
            // if the mouse leaves the control, we will not be beginning a drag drop operation
            allowDrag = false;
        }

        void WinGridDragDropRow_DragOver(object sender, System.Windows.Forms.DragEventArgs e)
        {
            // needed to change the drag drop to allow dropping
            e.Effect = DragDropEffects.Move;

            //Get the row from the Screen Point in EventArgs
            Point pointInGrid = this.PointToClient(new Point(e.X, e.Y));
            UIElement element = this.DisplayLayout.UIElement.ElementFromPoint(pointInGrid);
            UltraGridRow row = null;

            if (element != null)
            {
                RowUIElement rowElement = element.GetAncestor(typeof(RowUIElement)) as RowUIElement;
                if (rowElement != null)
                {
                    row = rowElement.Row;
                }
            }

            if (row == null)
            {
                //The mouse is not over a row, do not allowd dropping here
                e.Effect = DragDropEffects.None;
                //Erase any DropHighlight
                dFilter.ClearDropHighlight();
            }
            else
            {
                e.Effect = DragDropEffects.Move;
                dFilter.SetDropHighlight(row, pointInGrid);
            }


            //Logic to scroll the WinGrid if the cursor is dragged near the top or bottom of the control
            if (pointInGrid.Y < this.Height / 4)
            {
                this.DisplayLayout.RowScrollRegions[0].Scroll(RowScrollAction.LineUp);
            }
            if (pointInGrid.Y > this.Height - (this.Height / 10))
            {
                this.DisplayLayout.RowScrollRegions[0].Scroll(RowScrollAction.LineDown);
            }
        }

        void WinGridDragDropRow_DragLeave(object sender, EventArgs e)
        {
            // When the mouse goes outside the control, clear the drophighlight. 
            // Since the DropHighlight is cleared when the mouse is not over a WinGrid, anyway, this is probably not needed.
            // But, just in case the user goes from a ListViewItem directly off the control...
            dFilter.ClearDropHighlight();
        }

        void WinGridDragDropRow_DragDrop(object sender, System.Windows.Forms.DragEventArgs e)
        {
            // get the row being dropped onto if one exists 
            UltraGridRow dropTarget = dFilter.DropHighlightRow;

            // only if we have a row that we are dropping on do we do anything
            if (dropTarget != null)
            {
                int indexModifier = 0;
                UltraGridRow rowToMove = e.Data.GetData(typeof(UltraGridRow)) as UltraGridRow;

                //If dropTarget row is above the row being moved and the indicator appears below, we must mofify the move index
                if (dropTarget.Index < rowToMove.Index && dFilter.dropLinePosition == GridRowDropLinePositionEnum.BelowItem)
                {
                    indexModifier = 1;
                }
                //If dropTarget row is below the row being moved and the indicator appears above, we must mofify the move index
                if (dropTarget.Index > rowToMove.Index && dFilter.dropLinePosition == GridRowDropLinePositionEnum.AboveItem)
                {
                    indexModifier = -1;
                }

                //move the row to the new drop position
                this.Rows.Move(rowToMove, dropTarget.Index + indexModifier);

            }
            //Clear the drop indicator
            dFilter.SetDropHighlight(null, GridRowDropLinePositionEnum.None);
        }
        #endregion

        #region Methods

        protected override void OnCreateControl()
        {
            base.OnCreateControl();

            if (!this.DesignMode)
            {
                // Wire up events
                this.DragDrop += new System.Windows.Forms.DragEventHandler(WinGridDragDropRow_DragDrop);
                this.DragLeave += new EventHandler(WinGridDragDropRow_DragLeave);
                this.DragOver += new System.Windows.Forms.DragEventHandler(WinGridDragDropRow_DragOver);
                this.MouseDown += new MouseEventHandler(WinGridDragDropRow_MouseDown);
                this.MouseLeave += new EventHandler(WinGridDragDropRow_MouseLeave);
                this.MouseMove += new System.Windows.Forms.MouseEventHandler(WinGridDragDropRow_MouseMove);
                this.MouseUp += new System.Windows.Forms.MouseEventHandler(WinGridDragDropRow_MouseUp);

                dFilter.Invalidate += new EventHandler(dFilter_Invalidate);

                // Allow dropping on the control
                this.AllowDrop = true;

                dFilter.DropLineColor = Color.Blue;
                dFilter.DropLineWidth = 3;

                this.DrawFilter = dFilter;
            }
        }

        // determines if the mouse has moved a certain distance.
        private bool DragDistanceReached(Point p1, Point p2)
        {
            // get the x distance
            int xDist = Math.Abs(p1.X - p2.X);
            
            // get the y distance
            int yDist = Math.Abs(p1.Y - p2.Y);
            
            // get the total distance
            double totalDistance = Math.Sqrt(xDist * xDist + yDist * yDist);

            // we can drag if totalDistance is greater than three.
            return totalDistance > 3;
        }

        #endregion
    }
}
